rm(list=ls())
source(file = "./functions/functionsGeneral.R")
source(file = "./functions/functionR2Netcdf.R")
source(file = "./functions/functionNetcdf2R.R")
source(file = "./functions/functionConvert.R")
source(file = "./functions/infoGeneral.R")

submitscript <- FALSE

if (submitscript) {
  members      <- c(1:15)
  initYears    <- c(1981:2010)
  initMonths   <- c(X:X)
  locName      <- 'X'
  resolution   <- 'X'
  inPath       <- sprintf("../DATA/System4_seasonal_15_rev1.0/%s_%s", locName, resolution)
  outPath      <- sprintf("../DATA/System4_seasonal_15_rev1.2/%sdeg/%s_noBC", resolution, locName)
} else {
  members      <- c(1:2)
  initYears    <- c(1981:1982)
  initMonths   <- 1
  locName      <- "GHA"
  #locName      <- "EU"
  resolution   <- "0.75"
  inPath       <- sprintf("../DATA/System4_seasonal_15_rev1.0/%s_%s", locName, resolution)
  outPath      <- sprintf("../DATA/System4_seasonal_15_rev1.2/%sdeg/%s_noBC", resolution, locName)
}

mask<-Netcdf2R(inFile = sprintf("../DATA/mask/mask_wfdei_%s_%s.nc4", locName, resolution), "mask")

variables<-names(variableInfo)

print("start")
for (variableName in variables) {
  dir.create(sprintf("%s/%s", outPath, variableName), recursive = TRUE, showWarnings = FALSE)
  for (initMonth in initMonths) {
    for (iMember in members) {
      for (initYear in initYears) {
        if ('ecomsName' %in% names(variableInfo[[variableName]])) {
          variableNameECOMS<-variableInfo[[variableName]]$ecomsName
        } else {
          variableNameECOMS<-variableName
        }
        
        ## Load file...
        inFile <- sprintf("%s/%s/%s_forcing_seas15_%s_noBC_E%02d_%04d_%02d.RData", inPath, variableNameECOMS, variableNameECOMS, locName, iMember, initYear, initMonth)
        print(sprintf("Reading: %s",inFile))
        load(file = inFile)
        RData<-sys4
        rm(sys4)
        
        # Check units
        RData$Variable$varName<-variableName
        attr(RData$Variable,"standard_name")<-variableInfo[[variableName]]$standardName
        attr(RData$Variable,"long_name")<-variableInfo[[variableName]]$longName
        attr(RData$Variable,"units")<-variableInfo[[variableName]]$unitsEcoms
        
        RData$xyCoords$x[]<-round(RData$xyCoords$x[],2)
        RData$xyCoords$y[]<-round(RData$xyCoords$y[],2)
        
        ## set all negative precipitation values 0
        if (variableName == "pr") {
          RData$Data[RData$Data < 0] <- 0
        }
        
      ## Convert Units
      if (variableName == "rsds" || variableName == "rlds") {
        RData<-convert(RData, fromUnit = "1 W m-2", toUnit = variableInfo[[variableName]]$units)
      } else {
        RData<-convert(RData, toUnit = variableInfo[[variableName]]$units)
        # RData<-convert(RData, fromUnit = variableInfo[[variableName]]$unitsEcoms, toUnit = variableInfo[[variableName]]$units)
      }

        ## check if grids are the same as mask
        ## if not: make the same as mask
        if (RData$xyCoords$x[1] != locationInfo[[sprintf("res%s",resolution)]][[locName]]$lonmin ||
            RData$xyCoords$x[length(RData$xyCoords$x)] != locationInfo[[sprintf("res%s",resolution)]][[locName]]$lonmax ||
            RData$xyCoords$y[1] != locationInfo[[sprintf("res%s",resolution)]][[locName]]$latmin ||
            RData$xyCoords$y[length(RData$xyCoords$y)] != locationInfo[[sprintf("res%s",resolution)]][[locName]]$latmax) {
          
          indexesLat<-c(which(RData$xyCoords$y==mask$xyCoords$y[1]):which(RData$xyCoords$y==mask$xyCoords$y[length(mask$xyCoords$y)]))
          indexesLon<-c(which(RData$xyCoords$x==mask$xyCoords$x[1]):which(RData$xyCoords$x==mask$xyCoords$x[length(mask$xyCoords$x)]))
          RData$xyCoords$x<-RData$xyCoords$x[indexesLon]
          RData$xyCoords$y<-RData$xyCoords$y[indexesLat]
          attr(RData$Data,"projection") <- "+proj=longlat +ellps=WGS84 +datum=WGS84 +no_defs +towgs84=0,0,0"
          RData$Data<-RData$Data[,indexesLat,indexesLon]
          attr(RData$Data,"dimensions") <- c("time","lat","lon")
        }
        
        ## Apply mask
        for (iTime in 1:dim(RData$Data)[2]) {
          RData$Data[iTime,,]<-RData$Data[iTime,,]*mask$Data[1,,]
        }
        
        ## add some extra attributes
        attr(RData,"contact") <- "Wietse Franssen (wietse.franssen@wur.nl)"
        
        ## Save file...
        outPrefix <- sprintf("%s/%s/%s_forcing_seas15_%s_noBC_E%02d_INIT%04d_%02d", outPath, variableName, variableName, locName, iMember, initYear, initMonth)
        # print(outPrefix)
        R2Netcdf(paste0(outPrefix, ".nc4"), RData)
      }
    }
  }
}
