rm(list=ls())
library(WFRTools)

source(file = "./functions/infoGeneral.R")
source(file = "./functions/functionsGeneral.R")

submitscript <- FALSE

if (submitscript) {
  members    <- c(1:15)
  initYears <-c(1981:2010)
  targetMonths  <- c(X:X)
  leadMonths<-c(0:6)
  locName    <- 'X'
  resolution<-"0.50"
  inPathNoBC<-sprintf("../DATA/System4_seasonal_15_rev1.1/%sdeg/%s_noBC", resolution, locName)
  inPathBC<-sprintf("../DATA/System4_seasonal_15_rev1.1/%sdeg/%s_BC", resolution, locName)
  outPath<-"../Analysis_output/pics/checkBias_rev1.1_otherVars"
} else {
  members    <-c(1:15)
  initYears <-c(1981:2010)
  targetMonths<-c(1:12)
  leadMonths<-c(0:6)
  #locName<-"GHA"
  locName<-"EU"
  resolution<-"0.50"
  inPathNoBC<-sprintf("../DATA/System4_seasonal_15_rev1.1/%sdeg/%s_noBC", resolution, locName)
  inPathBC<-sprintf("../DATA/System4_seasonal_15_rev1.1/%sdeg/%s_BC", resolution, locName)
  outPath<-"../Analysis_output/pics/checkBias_rev1.1_otherVars"
}

variables<-names(variableInfo)
variables<-c( "pr"    ,  "psl"    , "rsds"   , "rlds" ,   "tasmin","tasmax", "huss" ,   "sfcWind")
variables<-c("sfcWind")
#variables<-c("pr")

dir.create(outPath, recursive = TRUE, showWarnings = FALSE)

print("start")
for (variableName in variables) {
  for (targetMonth in targetMonths) {
    for (leadMonth in leadMonths) {
      targetSYear <- getInitTargetInfo( initYear = initYears[1], targetMonth = targetMonth, leadMonth = leadMonth )$targetYear
      targetEYear <- getInitTargetInfo( initYear = initYears[length(initYears)], targetMonth = targetMonth, leadMonth = leadMonth )$targetYear

      ######## OPEN FILES #####################
      noBCFile <- sprintf("%s/%s_forcing_seas15_%s_noBC_E%02d-%02d_TAR%4d-%4d_%02d_LM%d.RData",
                          inPathNoBC, variableName, locName, 
                          members[1], members[length(members)],
                          targetSYear, targetEYear,
                          targetMonth, leadMonth)
      
      print(sprintf("Open: %s", noBCFile))
      load(noBCFile)
      RData_noBC<-RData
      rm(RData)
      
      BCFile <- sprintf("%s/%s_forcing_seas15_%s_BC_E%02d-%02d_TAR%4d-%4d_%02d_LM%d.RData",
                        inPathBC, variableName, locName, 
                        members[1], members[length(members)],
                        targetSYear, targetEYear,
                        targetMonth, leadMonth)
      print(sprintf("Open: %s", BCFile))
      load(BCFile)
      RData_BC<-RData
      rm(RData)      
    
    
      if (variableName == "pr" ) {
        obsFile <- sprintf("/home/WUR/frans004/L_BACKUP/PROJECTS/EUPORIAS/biascorrection/DATA/wfdei_rev3.0/0.50deg/%s/%s_wfdei_GPCC_EU_%4d-%4d_%02d.RData",
                        locName, variableName,  
                        initYears[1], initYears[length(initYears)],
                        targetMonth)
      } else {
        obsFile <- sprintf("/home/WUR/frans004/L_BACKUP/PROJECTS/EUPORIAS/biascorrection/DATA/wfdei_rev3.0/0.50deg/%s/%s_wfdei_EU_%4d-%4d_%02d.RData",
                        locName, variableName,  
                        initYears[1], initYears[length(initYears)],
                        targetMonth)
      }
      print(sprintf("Open: %s", obsFile))
      load(obsFile)
      obs<-RData
      rm(RData)
      
      #############################
      
      ## Check-Correction:
      barLimits<-c(colorbarLimits[[variableName]]$minValue,colorbarLimits[[variableName]]$maxValue)
      barLimitsDiff<-c(colorbarLimits[[variableName]]$diffMin,colorbarLimits[[variableName]]$diffMax)
      print("obs_mean...")
      obs_mean<-obs;obs_mean$Data<-apply(obs$Data, c(2,3), mean)
      p_obs<-ggplotje(obs_mean, title = paste0(variableInfo[[variableName]]$longName, " ", variableInfo[[variableName]]$units, "\nWFDEI (", month.name[targetMonth], ")"),barLimits = barLimits)
      #        p_obs<-ggplotje(obs_mean, title = paste0(variableInfo[[variableName]]$longName, " ", variableInfo[[variableName]]$units, "\nWFDEI (", month.name[targetMonth], ")"))
      print("BC_mean...")
      RData_BC_mean<-RData_BC;RData_BC_mean$Data<-apply(RData_BC$Data, c(1,3,4), mean)
      p_BC<-ggplotje(RData_BC_mean, title = paste0(variableInfo[[variableName]]$longName, " ", variableInfo[[variableName]]$units, "\nseas15_BC (", month.name[targetMonth], ", leadMonth:", leadMonth, ")"),barLimits = barLimits)
      #        p_BC<-ggplotje(RData_BC_mean, title = paste0(variableInfo[[variableName]]$longName, " ", variableInfo[[variableName]]$units, "\nseas15_BC (", month.name[targetMonth], ", LM:", leadMonth, "/SM:", iStartMonth, "/S:", iSeason, ")"))
      print("noBC_mean...")
      RData_noBC_mean<-RData_noBC;RData_noBC_mean$Data<-apply(RData_noBC$Data, c(1,3,4), mean)
      p_noBC<-ggplotje(RData_noBC_mean, title = paste0(variableInfo[[variableName]]$longName, " ", variableInfo[[variableName]]$units, "\nseas15_noBC (", month.name[targetMonth], ", leadMonth:", leadMonth, ")"),barLimits = barLimits)
      #        p_noBC<-ggplotje(RData_noBC_mean, title = paste0(variableInfo[[variableName]]$longName, " ", variableInfo[[variableName]]$units, "\nseas15_noBC (", month.name[targetMonth], ", LM:", leadMonth, "/SM:", iStartMonth, "/S:", iSeason, ")"))
      
      RData_BC_mean2<-RData_BC
      RData_BC_mean2$Data<-apply(RData_BC_mean$Data, c(2,3), mean)
      obsdiff<-obs_mean
      obsdiff$Data<-obs_mean$Data-RData_BC_mean2$Data
      #       p_obsVsBC<-ggplotje(obsdiff, title = "Observed vs BC")
#      p_obsVsBC<-ggplotje(obsdiff, title = paste0(variableInfo[[variableName]]$longName, " ", variableInfo[[variableName]]$units, "\nWFDEI - seas15_BC"),barLimits = barLimitsDiff,barDiff=TRUE,backgroudColor = "grey85")
      p_obsVsBC<-ggplotje(obsdiff, title = paste0(variableInfo[[variableName]]$longName, " ", variableInfo[[variableName]]$units, "\nWFDEI - seas15_BC"),backgroudColor = "grey85")
      #        p_obsVsBC<-ggplotje(obsdiff, title = paste0(variableInfo[[variableName]]$longName, " ", variableInfo[[variableName]]$units, "\nWFDEI - seas15_BC"),barDiff=TRUE,backgroudColor = "grey85")
      
      RData_noBC_mean2<-RData_BC
      RData_noBC_mean2$Data<-apply(RData_noBC_mean$Data, c(2,3), mean)
      obsdiffnoBC<-obs_mean
      obsdiffnoBC$Data<-obs_mean$Data-RData_noBC_mean2$Data
      #          p_obsVsnoBC<-ggplotje(obsdiffnoBC, title = "Observed vs noBC")
#      p_obsVsnoBC<-ggplotje(obsdiffnoBC, title = paste0(variableInfo[[variableName]]$longName, " ", variableInfo[[variableName]]$units, "\nWFDEI - seas15_noBC"),barLimits = barLimitsDiff,barDiff=TRUE,backgroudColor = "grey85")
      p_obsVsnoBC<-ggplotje(obsdiffnoBC, title = paste0(variableInfo[[variableName]]$longName, " ", variableInfo[[variableName]]$units, "\nWFDEI - seas15_noBC"),backgroudColor = "grey85")
      #        p_obsVsnoBC<-ggplotje(obsdiffnoBC, title = paste0(variableInfo[[variableName]]$longName, " ", variableInfo[[variableName]]$units, "\nWFDEI - seas15_noBC"),barDiff=TRUE,backgroudColor = "grey85")
      
      
      print("densPlots...")
      print(dim(obs$Data))
      print(dim(RData_BC$Data))
      print(dim(RData_noBC$Data))
      print(dim(RData_noBC$Data))
      
      densPlot <-function(lon = NULL, lat = NULL) {
        data_obs<-obs$Data[,lat==obs$xyCoords$y,lon==obs$xyCoords$x]
        data_BC<-RData_BC$Data[members[1]:members[length(members)],,lat==obs$xyCoords$y,lon==obs$xyCoords$x]
        data_noBC<-RData_noBC$Data[members[1]:members[length(members)],,lat==obs$xyCoords$y,lon==obs$xyCoords$x]
        
        dat <- data.frame(legend = factor(c(rep("1: OBS",length(as.vector(data_obs))),
                                            rep("3: noBC",length(as.vector(data_noBC))),
                                            rep("2: BC",length(as.vector(data_BC))))), 
                          data = c(as.vector(data_obs),
                                   as.vector(data_noBC),
                                   as.vector(data_BC)))
        nBins<-20
        bins<-seq(min(dat$data),max(dat$data),(max(dat$data)-min(dat$data))/(nBins+1))
        # Density plots
        #  pplot<-ggplot(dat, aes(x=data, colour=legend, fill=legend)) + 
        pplot<-ggplot(dat, aes(x=data, fill=legend)) + 
          #geom_histogram(binwidth=1, position="dodge",aes(y = ..density..)) +
          geom_histogram(breaks=bins, position="dodge",aes(y = ..density..)) +
          #geom_density() +
          ggtitle(paste0("lon: ",lon, " lat: ", lat))
        return(pplot)
      }
      
      if (locName == "GHA") {
        p4<-densPlot(30.25,-2.75)
        p5<-densPlot(37.25,-2.75)
        p6<-densPlot(37.25,10.75)
      } else {
        #          p4<-densPlot(-5.25,40.75)
        #          p5<-densPlot(30.25,50.75)
        #          p6<-densPlot(10.25,46.25)
        p4<-densPlot(19.25,54.25)
        p5<-densPlot(27.25,43.75)
        p6<-densPlot(10.25,46.25)
      }
      
      nameFilePicture <- sprintf("%s/%s_%s_E%02d-%02d_TAR%4d-%4d_%02d_LM%d",
                          outPath, variableName, locName, 
                          members[1], members[length(members)],
                          initYears[1], initYears[length(initYears)],
                          targetMonth, leadMonth)
                          
      print(paste0("Saving picture : ", nameFilePicture))
      
      png(paste0(nameFilePicture,'.png'), width = 1920, height = 1080)
      multiplot(p_obs, p_noBC, p_BC, p_obs, p_obsVsnoBC, p_obsVsBC, p4 ,p5, p6, cols=3)
      
      dev.off()
    }
  }
}
