rm(list=ls())
library(ecomsUDG.Raccess)
library(downscaleR)
library(MASS)

source(file = "./functions/functionsGeneral.R")
source(file = "./functions/infoGeneral.R")
source(file = "./functions/functionMakeSelection.R")
source(file = "./functions/functionBiascorrectionWUR.R")
source(file = "./functions/functionR2Netcdf.R")
source(file = "./functions/functionNetcdf2R.R")

submitscript <- FALSE

if (submitscript) {
  members      <- c(1:15)
  initYears    <- c(1981:2010)
  targetMonths <- c(X:X)
  leadMonths   <- c(0:6)
  locName      <- 'X'
  resolution   <- 'X'
  obsPath      <- sprintf("../DATA/wfdei_rev3.0/%sdeg/%s/", resolution, locName)
  inPath       <- sprintf("../DATA/System4_seasonal_15_rev1.1/%sdeg/%s_noBC", resolution, locName)
  outPath      <- sprintf("../DATA/System4_seasonal_15_rev1.1/%sdeg/%s_BC", resolution, locName)
} else {
  members      <- c(1:15)
  initYears    <- c(1981:2010)
  targetMonths <- 6
  leadMonths   <- c(0:6)
  #locName      <- "GHA"
  locName      <- "EU"
  resolution   <- "0.50"
  obsPath      <- sprintf("../DATA/wfdei_rev3.0/%sdeg/%s/", resolution, locName)
  inPath       <- sprintf("../DATA/System4_seasonal_15_rev1.1/%sdeg/%s_noBC", resolution, locName)
  outPath      <- sprintf("../DATA/System4_seasonal_15_rev1.2_1.0mm_adapt/%sdeg/%s_BC", resolution, locName)
}

variables<-names(variableInfo)
variables<-c( "tas", "tasmin", "tasmax", "rsds", "rlds", "pr", "huss", "sfcWind")
variables<-c( "tas", "tasmin", "tasmax", "rsds", "rlds", "huss", "sfcWind")

dir.create(outPath, recursive = TRUE, showWarnings = FALSE)

print("start GPCC")
for (variableName in variables) {
  for (targetMonth in targetMonths) {
    for (leadMonth in leadMonths) {
      targetSYear <- getInitTargetInfo( initYear = initYears[1], targetMonth = targetMonth, leadMonth = leadMonth )$targetYear
      targetEYear <- getInitTargetInfo( initYear = initYears[length(initYears)], targetMonth = targetMonth, leadMonth = leadMonth )$targetYear
      
      iPrefix <- sprintf("%s/%s_forcing_seas15_%s_noBC_E%02d-%02d_TAR%4d-%4d_%02d_LM%d",
                       inPath, variableName, locName, 
                       members[1], members[length(members)],
                       targetSYear, targetEYear,
                       targetMonth, leadMonth)

      print(sprintf("Open: %s", iPrefix))
      RData_noBC <- Netcdf2R(paste0(iPrefix, ".nc4"), variableName)

      #load(file = paste0(iPrefix, ".RData"))
      #RData_noBC<-RData
      #rm(RData)
      
      if (variableName == "pr") {
        obsFile <- sprintf("%s/%s_wfdei_GPCC_%s_%4d-%4d_%02d.RData",
                          obsPath, variableName, locName, 
                          initYears[1], initYears[length(initYears)],
                          targetMonth)
#        obsFile <- sprintf("%s/%s_wfdei_%s_%4d-%4d_%02d.RData",
#                          obsPath, variableName, locName, 
#                          initYears[1], initYears[length(initYears)],
#                          targetMonth)
      } else {
        obsFile <- sprintf("%s/%s_wfdei_%s_%4d-%4d_%02d.RData",
                          obsPath, variableName, locName, 
                          initYears[1], initYears[length(initYears)],
                          targetMonth)
      }
      print(sprintf("Open: %s", obsFile))
      load(obsFile)

      RData_obs<-RData
      rm(RData)
#      newGrid<-getGrid(RData_noBC)
#      indexesLat<-c(which(RData$xyCoords$y==newGrid$y[1]):which(RData$xyCoords$y==newGrid$y[2]))
#      indexesLon<-c(which(RData$xyCoords$x==newGrid$x[1]):which(RData$xyCoords$x==newGrid$x[2]))
#      RData_new$xyCoords<-RData_noBC$xyCoords
#      RData_new$Data<-RData$Data[,indexesLat,indexesLon]
#      attr(RData_new$Data,"dimensions") <- c("time","lat","lon")
#      RData<-RData_new
#      rm(RData_new)
      
      ## Make uniform and match the selections
      dates1<-as.character(as.POSIXct(RData_obs$Dates$start, tz = "GMT"))
      dates2<-as.character(as.POSIXct(RData_noBC$Dates$start, tz = "GMT"))
      overlapping_dates<-as.Date(intersect(dates1, dates2))
      rm(dates2,dates1)

      RData_obs <- subsetRData(RData_obs, dates = overlapping_dates)
      RData_noBC_pred <- subsetRData(RData_noBC, dates = overlapping_dates)
      
      if (variableName == "pr") {
        ## Correction: PRECIP
        print(sprintf("Bias correction..."))
        RData <- biasCorrect(RData_obs, RData_noBC, nbin = 100, epsilon = 0.00001)
      } else {
        ## Correction: other variables
        print(sprintf("Bias correction..."))
        RData <- biasCorrection (RData_obs, RData_noBC_pred, RData_noBC, method = "qqmap", multi.member = FALSE) # bias
      }

      ## Write output
      oPrefix <- sprintf("%s/%s_forcing_seas15_%s_BC_E%02d-%02d_TAR%4d-%4d_%02d_LM%d",
                       outPath, variableName, locName, 
                       members[1], members[length(members)],
                       targetSYear, targetEYear,
                       targetMonth, leadMonth)
      
      print(sprintf("Saving: %s", oPrefix))
      #save(file = paste0(oPrefix, ".RData"), RData)
      R2Netcdf(paste0(oPrefix, ".nc4"), RData)
    }
  }
}
