rm(list=ls())
library(fields) # e.g: using the fields library
library(abind)
library(ncdf4)
library(ecomsUDG.Raccess)
source(file = "./functions/infoGeneral.R")
source(file = "./functions/functionsGeneral.R")
source(file = "./functions/functionR2Netcdf.R")

loginECOMS_UDG(username = "wietsefranssen", password = "ECOMS")

submitscript <- FALSE

if (submitscript) {
  members    <- c(1:15)
#  members    <- c(1:1)
  targetMonths <- c(1:12)
#  targetMonths <- c(1)
  targetYears  <- c(1981:2011)
#  targetYears  <- c(1981:1982)
  leadMonths  <- c(X:X)
  locName    <- 'X'
#  outPath<-"../DATA/System4_seasonal_15/0.75deg/EU_noBC"
  outPath<-sprintf("../DATA/System4_seasonal_15_rev3.0/0.75deg/%s_noBC", locName)
} else {
  members    <-c(1,2)
  targetMonths <-c(1:2)
  targetYears  <-c(2009:2011)
  targetYears  <-c(1981:1982)
  leadMonths  <-c(0:1)
  locName<-"GHA"
  # locName<-"EU"
  outPath <- "./testData/noBC/"
  outPath<-"./testData/newDownload"
}

variables<-names(variableInfo)
#variables<-c("pr", "rsds", "rlds")
#variables<-c( "sfcWind", "psl" , "huss", "tas", "tasmin", "tasmax")
#variables<-c("tas", "rsds", "rlds", "pr")

dir.create(outPath, recursive = TRUE, showWarnings = FALSE)

for (variableName in variables) {
  for (targetMonth in targetMonths) {
    for (leadMonth in leadMonths) {
      ## get the available target years for the current targetMonth and leadMonth
      availableTargetYears <- getAvailableTargetYears(targetYears = targetYears, targetMonth = targetMonth, leadMonth = leadMonth)
      nYears <- length(availableTargetYears)
      
      if ('ecomsName' %in% names(variableInfo[[variableName]])) {
        variableNameECOMS<-variableInfo[[variableName]]$ecomsName
      } else {
        variableNameECOMS<-variableName
      }
      
      
      print(sprintf("targetMonth: %s, leadMonth: %s, years: %d-%d",month.name[targetMonth], leadMonth, availableTargetYears[1], availableTargetYears[nYears]))
      
      oPrefix <- sprintf("%s/%s_forcing_seas15_%s_noBC_E%02d-%02d_%4d-%4d_%02d_LM%d",
                         outPath, variableName, locName, 
                         members[1], members[length(members)],
                         availableTargetYears[1], availableTargetYears[nYears],
                         targetMonth, leadMonth)
      print(oPrefix)
      
      RData<-loadECOMS(dataset = "System4_seasonal_15", 
                       var = variableNameECOMS,
                       members = members, 
                       lonLim = c(locationInfo$res0.75[[locName]]$lonmin, locationInfo$res0.75[[locName]]$lonmax),
                       latLim = c(locationInfo$res0.75[[locName]]$latmin, locationInfo$res0.75[[locName]]$latmax), 
                       season = targetMonth, 
                       years = availableTargetYears, 
                       leadMonth = leadMonth,
                       time = "DD")
      #     print(RData$InitializationDates)
      #     print(RData$Dates$start)
      
      RData$xyCoords$x[]<-round(RData$xyCoords$x[],2)
      RData$xyCoords$y[]<-round(RData$xyCoords$y[],2)
      
      # Check units
      RData$Variable$varName<-variableName
      attr(RData$Variable,"standard_name")<-variableInfo[[variableName]]$standardName
      attr(RData$Variable,"long_name")<-variableInfo[[variableName]]$longName
      attr(RData$Variable,"units")<-variableInfo[[variableName]]$unitsEcoms
      
      ## add some extra attributes
      attr(RData,"contact") <- "Wietse Franssen (wietse.franssen@wur.nl)"
      
      save(file = sprintf("%s.RData",oPrefix), RData)
      # R2Netcdf(sprintf("%s.nc4",oPrefix), RData)
    }
  }
}
