rm(list=ls())
library(downscaleR)

source(file = "./functions/functionsGeneral.R")
source(file = "./functions/infoGeneral.R")

submitscript <- FALSE

if (submitscript) {
  members    <- c(1:15)
  initYears <-c(1981:2010)
  targetMonths <- c(X:X)
  leadMonths<-c(0:6)
  locName    <- 'X'
  fromResolution <- "0.75"
  toResolution <- "0.50"
  inPath<-sprintf("../DATA/System4_seasonal_15_rev1.1TAR_old/%sdeg/%s_noBC", fromResolution, locName)
  outPath<-sprintf("../DATA/System4_seasonal_15_rev1.1TAR_old/%sdeg/%s_noBC", toResolution, locName)
} else {
  members    <-c(1:15)
  initYears <-c(1981:2010)
  targetMonths<-1
  leadMonths<-c(0:6)
  #locName<-"GHA"
  locName<-"EU"
  fromResolution <- "0.75"
  toResolution <- "0.50"
  inPath<-sprintf("../DATA/System4_seasonal_15_rev1.1/%sdeg/%s_noBC", fromResolution, locName)
  outPath<-sprintf("../DATA/System4_seasonal_15_rev1.1/%sdeg/%s_noBC", toResolution, locName)
}

variables<-names(variableInfo)
variables<-c("tasmin", "tasmax", "pr", "psl", "rsds", "rlds", "huss", "sfcWind" )
dir.create(outPath, recursive = TRUE, showWarnings = FALSE)

print("start")
for (variableName in variables) {
  for (targetMonth in targetMonths) {
    for (leadMonth in leadMonths) {
      targetSYear <- getInitTargetInfo( initYear = initYears[1], targetMonth = targetMonth, leadMonth = leadMonth )$targetYear
      targetEYear <- getInitTargetInfo( initYear = initYears[length(initYears)], targetMonth = targetMonth, leadMonth = leadMonth )$targetYear
      
      iFile <- sprintf("%s/%s_forcing_seas15_%s_noBC_E%02d-%02d_TAR%4d-%4d_%02d_LM%d.RData",
                       inPath, variableName, locName, 
                       members[1], members[length(members)],
                       targetSYear, targetEYear,
                       targetMonth, leadMonth)

      print(sprintf("Open: %s", iFile))
      load(iFile)
      
      newGrid<-getGrid(RData)
      newGrid$x <- c(locationInfo$res0.50[[locName]]$lonmin, locationInfo$res0.50[[locName]]$lonmax)
      newGrid$y <- c(locationInfo$res0.50[[locName]]$latmin, locationInfo$res0.50[[locName]]$latmax)
      attr(newGrid,"resX") <- 0.50
      attr(newGrid,"resY") <- 0.50
      
      print(sprintf("Regridding..."))
      RData <- interpGridData(RData, new.grid = newGrid, method = "bilinear");
#      RData <- interpGridData(RData, new.grid = newGrid, method = "nearest");
      
      ## set all negative precipitation values 0
      if (variableName == "pr") {
        RData$Data[RData$Data < 0] <- 0
      }
      
      oFile <- sprintf("%s/%s_forcing_seas15_%s_noBC_E%02d-%02d_TAR%4d-%4d_%02d_LM%d.RData",
                       outPath, variableName, locName, 
                       members[1], members[length(members)],
                       targetSYear, targetEYear,
                       targetMonth, leadMonth)
      
      print(sprintf("Saving: %s", oFile))
      save(file= oFile, RData)
    }
  }
}
