rm(list=ls())
library(fields) # e.g: using the fields library
library(abind)
library(ncdf4)
library(ecomsUDG.Raccess)
source(file = "./functions/functionsGeneral.R")
source(file = "./functions/functionReformat.R")
source(file = "./functions/functionR2Netcdf.R")
source(file = "./functions/infoGeneral.R")

submitscript <- FALSE

if (submitscript) {
  members      <- c(1:15)
  initYears    <- c(1981:2010)
  initMonths   <- c(X:X)
  locName      <- 'X'
  resolution   <- 'X'
  inPath       <- sprintf("../DATA/System4_seasonal_15_rev1.0/%s_%s", locName, resolution)
  outPath      <- sprintf("../DATA/System4_seasonal_15_rev1.1/%sdeg/%s_noBC", resolution, locName)
} else {
  members      <- c(1:2)
  initYears    <- c(1981:1982)
  initMonths   <- 1
  #locName      <- "GHA"
  locName      <- "EU"
  resolution   <- "0.75"
  inPath       <- sprintf("../DATA/System4_seasonal_15_rev1.0/%s_%s", locName, resolution)
  outPath      <- sprintf("../DATA/System4_seasonal_15_rev1.x/%sdeg/%s_noBC", resolution, locName)
}
mask<-Netcdf2R(inFile = sprintf("../DATA/mask/mask_wfdei_%s.nc4", locName),"mask")

variables<-names(variableInfo)

dir.create(outPath, recursive = TRUE, showWarnings = FALSE)

leadMonths<-c(0:6)

print("start")
for (initMonth in initMonths) {
  for (variableName in variables) {
    if ('ecomsName' %in% names(variableInfo[[variableName]])) {
      variableNameECOMS<-variableInfo[[variableName]]$ecomsName
    } else {
      variableNameECOMS<-variableName
    }
    inFile <- sprintf("%s/%s/%s_forcing_seas15_%s_noBC_E<MEMBERS>_<YEARS>_%02d.RData", inPath, variableNameECOMS, variableNameECOMS, locName, initMonth)
    print(inFile)
    RDataAllLeadMonths <- reformat2Bias(members, initYears, initMonth, variableNameECOMS, locName, iFile = inFile)
    
    for (leadMonth in leadMonths) {
      targetMonth <- getInitTargetInfo( initYear = 0000, initMonth = initMonth, leadMonth = leadMonth )$targetMonth
      targetSYear <- getInitTargetInfo( initYear = initYears[1], initMonth = initMonth, leadMonth = leadMonth )$targetYear
      targetEYear <- getInitTargetInfo( initYear = initYears[length(initYears)], initMonth = initMonth, leadMonth = leadMonth )$targetYear
      print(sprintf("targetmonth: %s, leadMonth: %s",targetMonth, leadMonth))
      
      oPrefix <- sprintf("%s/%s_forcing_seas15_%s_noBC_E%02d-%02d_TAR%4d-%4d_%02d_LM%d",
                         outPath, variableName, locName, 
                         members[1], members[length(members)],
                         targetSYear, targetEYear,
                         targetMonth, leadMonth)
      print(oPrefix)
      
      RData<-RDataAllLeadMonths[[paste0("LeadMonth_",leadMonth)]]
      #         print(RData$InitializationDates)
      #         print(RData$Dates$start)
      
      ## set all negative precipitation values 0
      if (variableName == "pr") {
        RData$Data[RData$Data < 0] <- 0
      }
      
      RData$xyCoords$x[]<-round(RData$xyCoords$x[],2)
      RData$xyCoords$y[]<-round(RData$xyCoords$y[],2)
      
#      ## check if grids are the same
#      if (RData$xyCoords$x[1] != locationInfo$res0.50[[locName]]$lonmin ||
#          RData$xyCoords$x[length(RData$xyCoords$x)] != locationInfo$res0.50[[locName]]$lonmax ||
#          RData$xyCoords$y[1] != locationInfo$res0.50[[locName]]$latmin ||
#          RData$xyCoords$y[length(RData$xyCoords$y)] != locationInfo$res0.50[[locName]]$latmax) {
#
#        ## make grid
#        newGrid<-getGrid(RData)
#        newGrid$x <- c(locationInfo$res0.50[[locName]]$lonmin, locationInfo$res0.50[[locName]]$lonmax)
#        newGrid$y <- c(locationInfo$res0.50[[locName]]$latmin, locationInfo$res0.50[[locName]]$latmax)
#        attr(newGrid,"resX") <- 0.50
#        attr(newGrid,"resY") <- 0.50
#        ## regrid
#        print(sprintf("Regridding..."))
#        RData <- interpGridData(RData, new.grid = newGrid, method = "bilinear");
#      }

      ## Apply mask
      print("Apply mask")
      for (iMember in 1:members) {
        for (iTime in 1:dim(RData$Data)[2]) {
          RData$Data[iMember,iTime,,]<-RData$Data[iMember,iTime,,]*mask$Data[1,,]
        }
      }      

      
      # Check units
      RData$Variable$varName<-variableName
      attr(RData$Variable,"standard_name")<-variableInfo[[variableName]]$standardName
      attr(RData$Variable,"long_name")<-variableInfo[[variableName]]$longName
      attr(RData$Variable,"units")<-variableInfo[[variableName]]$unitsEcoms
      
      ## add some extra attributes
      attr(RData,"contact") <- "Wietse Franssen (wietse.franssen@wur.nl)"
      
      #save(RData, file = paste0(oPrefix, ".RData"))
      R2Netcdf(paste0(oPrefix, ".nc4"), RData)
    }
  }
}
