rm(list=ls())
library(fields) # e.g: using the fields library
library(abind)
library(ncdf4)
library(ecomsUDG.Raccess)
source(file = "./functions/functionsGeneral.R")
source(file = "./functions/functionReformat.R")
source(file = "./functions/functionR2Netcdf.R")
source(file = "./functions/functionNetcdf2R.R")
source(file = "./functions/functionConvert.R")
source(file = "./functions/infoGeneral.R")
showMemoryUse <- function(sort="size", decreasing=FALSE, limit) {

  objectList <- ls(parent.frame())

  oneKB <- 1024
  oneMB <- 1048576
  oneGB <- 1073741824

  memoryUse <- sapply(objectList, function(x) as.numeric(object.size(eval(parse(text=x)))))

  memListing <- sapply(memoryUse, function(size) {
        if (size >= oneGB) return(paste(round(size/oneGB,2), "GB"))
        else if (size >= oneMB) return(paste(round(size/oneMB,2), "MB"))
        else if (size >= oneKB) return(paste(round(size/oneKB,2), "kB"))
        else return(paste(size, "bytes"))
      })

  memListing <- data.frame(objectName=names(memListing),memorySize=memListing,row.names=NULL)

  if (sort=="alphabetical") memListing <- memListing[order(memListing$objectName,decreasing=decreasing),] 
  else memListing <- memListing[order(memoryUse,decreasing=decreasing),] #will run if sort not specified or "size"

  if(!missing(limit)) memListing <- memListing[1:limit,]

  print(memListing, row.names=FALSE)
  return(invisible(memListing))
}
submitscript <- FALSE

if (submitscript) {
  members      <- c(1:15)
  initYears    <- c(1981:2010)
  initMonths   <- c(X:X)
  locName      <- 'X'
  resolution   <- 'X'
  bcInfo       <- 'X'
  inPath       <- sprintf("../DATA/System4_seasonal_15_rev1.1/%sdeg/%s_%s", resolution, locName, bcInfo)
  outPath      <- sprintf("../DATA/System4_seasonal_15_rev1.1/%sdeg/%s_%s_modelFormat_perVariable", resolution, locName, bcInfo)
} else {
  members      <- c(1:15)
  initYears    <- c(1981:2010)
  initMonths   <- c(1:12)
  #locName      <- "GHA"
  locName      <- "EU"
  resolution   <- "0.50"
#  bcInfo       <- "BC"
  bcInfo       <- "noBC"
  inPath       <- sprintf("../DATA/System4_seasonal_15_rev1.1/%sdeg/%s_%s", resolution, locName, bcInfo)
  outPath      <- sprintf("../DATA/System4_seasonal_15_rev1.1/%sdeg/%s_%s_modelFormat_perVariable", resolution, locName, bcInfo)
}
mask<-Netcdf2R(inFile = sprintf("../DATA/mask/mask_wfdei_%s.nc4", locName),"mask")

variables<-names(variableInfo)
variables<-c( "rsds", "rlds", "pr", "tas", "tasmin", "tasmax", "huss", "sfcWind")
variables<-c( "rsds", "rlds")

leadMonths<-c(0:6)

sYear<-initYears[1]
eYear<-initYears[length(initYears)]
nYears<-length(initYears)

sMember<-members[1]
eMember<-members[length(members)]
nMembers<-length(members)


print("start")
for (initMonth in initMonths) {
  for (variableName in variables) {
    outPathTmp<-sprintf("%s/%s/", outPath, variableName)
    dir.create(outPathTmp, recursive = TRUE, showWarnings = FALSE)
    allocatedOutputArray<-FALSE
    
    for (leadMonth in leadMonths) {
      indexesYearsLeadMonths<-indexesForYearsPerLeadMonth(sYear,eYear,initMonth)
      
      targetMonth <- getInitTargetInfo( initYear = 0000, initMonth = initMonth, leadMonth = leadMonth )$targetMonth
      targetSYear <- getInitTargetInfo( initYear = initYears[1], initMonth = initMonth, leadMonth = leadMonth )$targetYear
      targetEYear <- getInitTargetInfo( initYear = initYears[length(initYears)], initMonth = initMonth, leadMonth = leadMonth )$targetYear
      #   print(sprintf("targetmonth: %s, leadMonth: %s",targetMonth, leadMonth))
      
      iPrefix <- sprintf("%s/%s_forcing_seas15_%s_%s_E%02d-%02d_TAR%4d-%4d_%02d_LM%d",
                         inPath, variableName, locName, bcInfo,
                         members[1], members[length(members)],
                         targetSYear, targetEYear,
                         targetMonth, leadMonth)
      print(iPrefix)
      #load(sprintf("%s.RData",iPrefix))
      RData<-Netcdf2R(sprintf("%s.nc4",iPrefix),variableName)
#      gc(TRUE)
      
      ## Apply mask
      print("Apply mask")
      for (iMember in 1:members) {
        for (iTime in 1:dim(RData$Data)[2]) {
          RData$Data[iMember,iTime,,]<-RData$Data[iMember,iTime,,]*mask$Data[1,,]
        }
      }      
#      gc(reset = TRUE)
#      showMemoryUse(decreasing=TRUE, limit=5)
      ## Convert Units
      if (variableName == "rsds" || variableName == "rlds") {
        RData<-convert(RData, fromUnit = "1 W m-2", toUnit = variableInfo[[variableName]]$units)
      } else {
        RData<-convert(RData, toUnit = variableInfo[[variableName]]$units)
      }
#      gc(reset = TRUE)
#      showMemoryUse(decreasing=TRUE, limit=5)
      if (!allocatedOutputArray) {
        ## allocate arrays
        finalList<-NULL
        listNames<-NULL
        for (iLM in 1:7) {
          finalList[[iLM]] <- RData
          listNames<-c(listNames,paste0("LeadMonth_",iLM-1))
          totalDays<-indexesYearsLeadMonths$eIndexes[as.character(eYear),iLM]
          
          finalList[[iLM]]$Data<-array(NA,dim=c(nMembers,totalDays,length(RData$xyCoords$y),length(RData$xyCoords$x)))
          finalList[[iLM]]$Dates$start<-character(length=totalDays)
          finalList[[iLM]]$Dates$end<-character(length=totalDays)
        }
        names(finalList)<-listNames
        allocatedOutputArray<-TRUE
      }
      finalList[[paste0("LeadMonth_",leadMonth)]]<-RData
#      gc(reset = TRUE)
#      showMemoryUse(decreasing=TRUE, limit=5)
    }
    
    for (initYear in initYears) {
      indexes<-indexesOfDaysPerMonth(initMonth, 7, initYear = initYear)
      
      ## Init structure
      RData<-finalList[["LeadMonth_0"]]
      RData$Data<-array(NA,dim=c(indexes[7,"eIndex"],length(RData$xyCoords$y),length(RData$xyCoords$x)))
      attr(RData$Data,"dimensions")<-c("time", "lat", "lon")
      RData$Dates$start<-array("",dim=c(indexes[7,"eIndex"]))
      RData$Dates$end<-array("",dim=c(indexes[7,"eIndex"]))

      for (iMember in members) {
        ## Fill with data
        for (iLM in 1:7) {
          #print(iLM)
          month<-indexes[iLM,"month"]
          year<-indexes[iLM,"year"]
          datesInCurrMonth <- seq(as.Date(as.Date(sprintf("%04d-%02d-01", year, month))),by = "1 day", length=indexes[iLM,"nDaysTotalInMonth"])
          iDates<-which(is.element(as.Date(finalList[[iLM]]$Dates$start),datesInCurrMonth),TRUE)
          RData$Dates$start[indexes[iLM,"sIndex"]:indexes[iLM,"eIndex"]]<-finalList[[iLM]]$Dates$start[iDates]
          RData$Dates$end[indexes[iLM,"sIndex"]:indexes[iLM,"eIndex"]]<-finalList[[iLM]]$Dates$end[iDates]
          RData$Data[indexes[iLM,"sIndex"]:indexes[iLM,"eIndex"],,]<-finalList[[iLM]]$Data[iMember,iDates,,]
        }
        RData$Members<-finalList[[iLM]]$Members[which(initYears==initYear)]
        RData$InitializationDates<-finalList[[iLM]]$InitializationDates[which(initYears==initYear)]

        ## Save to file...
        outFile <- sprintf("%s/%s_forcing_seas15_%s_%s_E%02d_%04d_%02d", outPathTmp, variableName, locName, bcInfo, iMember, initYear, initMonth)
        #print(outFile)
        R2Netcdf(outFile = sprintf("%s.nc4",outFile), RData)
      }
    }
  }
}
